@@ -1,18 +1,24 @@
+## create a named cell grob
+## a function of an integer index that returns a named grob
 
-textii <- function(d, gp=gpar(), name="row-label-", just="center"){
+textii <- function(d, gp=gpar(), name="row-label-",
+                   just="center", parse=TRUE){
   x <- switch(just, "center"=0.5, "right"=0.95, "left"=0.1)
-##   allow the correct space to fit well in a rectangle
-  function(ii)
-textGrob(x=x, label=d[ii], just=just, gp=gp, name=paste(name, ii, sep=""))
+  parseglobal <- parse
+  ##   allow the correct space to fit well in a rectangle
+  function(ii, parse=parseglobal){
+    lab <- if(parse) parse(text=d[ii]) else d[ii]
+    textGrob(x=x, label=lab, just=just, gp=gp, name=paste(name, ii, sep=""))
+  }
 }
 
 rectii <- function(ii, gp, name="row-fill-"){
   function(ii)
-rectGrob(gp=gp[[ii]], name=paste(name, ii, sep=""))
+    rectGrob(gp=gp[[ii]], name=paste(name, ii, sep=""))
 }
 
 makeTableGrobs <- function(content, rnames=NULL, cnames=NULL,
-        nrow, ncol,
+        nrow, ncol, parse=TRUE,
         row.just="center", col.just="center", core.just="center",
         equal.width = FALSE, equal.height=FALSE, 
         gpar.coretext = gpar(col="black", cex=1),
@@ -30,9 +36,9 @@ makeTableGrobs <- function(content, rnames=NULL, cnames=NULL,
 
  
  ## define some functions to generate named grobs
- makeOneRowname <- textii(d=rnames, gp=gpar.rowtext, name="row-label-", just=row.just)
- makeOneColname <- textii(d=cnames, gp=gpar.coltext, name="col-label-", just=col.just)
- makeOneLabel <- textii(d=content, gp=gpar.coretext, name="core-label-", just=core.just)
+ makeOneRowname <- textii(d=rnames, gp=gpar.rowtext, name="row-label-", just=row.just, parse=parse)
+ makeOneColname <- textii(d=cnames, gp=gpar.coltext, name="col-label-", just=col.just, parse=parse)
+ makeOneLabel <- textii(d=content, gp=gpar.coretext, name="core-label-", just=core.just, parse=parse)
  
 
  gp.corefillee <- gp.corefilleo <- gp.corefilloe <- gp.corefilloo <- gpar.corefill
@@ -48,7 +54,7 @@ makeTableGrobs <- function(content, rnames=NULL, cnames=NULL,
  gp.rowfille <- gp.rowfillo <- gpar.rowfill
  gp.rowfille[["alpha"]] <-  h.even.alpha
  gp.rowfillo[["alpha"]] <-  h.odd.alpha
-## print(nrow)
+
  gpar.rowfill <- rep(c(list(gp.rowfille), list(gp.rowfillo)), nrow)
  
  gp.colfille <- gp.colfillo <- gpar.colfill
@@ -107,11 +113,9 @@ makeTableGrobs <- function(content, rnames=NULL, cnames=NULL,
 
 
 arrangeTableGrobs <- function(lgt, lgf, nrow, ncol, widths, heights,
-                              show.colnames, show.rownames, 
-                              padding.h = unit(4, "mm"), padding.v=unit(4, "mm"), 
-                              just=c("center", "center"), separator= "white",
-                              show.vlines=FALSE, show.hlines=FALSE, show.namesep=FALSE, 
-                              show.box=FALSE, show.csep=FALSE, show.rsep=FALSE){
+                               padding.h = unit(4, "mm"), padding.v=unit(4, "mm"), 
+                               just=c("center", "center"), separator= "white",
+                               show.box=FALSE, show.csep=FALSE, show.rsep=FALSE){
  
   label.ind <- 1   # index running accross labels
 
@@ -128,39 +132,23 @@ arrangeTableGrobs <- function(lgt, lgf, nrow, ncol, widths, heights,
     }
   }
 
-  if(show.hlines){
-    ## draw horizontal lines, stopping or not before the names
-    for (ii in seq(2, nrow, 1)) 
+  ## draw horizontal lines, stopping or not before the names
+  for (ii in seq(1, nrow, 1)) 
       grid.segments(0, 0, 1, 0, gp=gpar(col=separator), vp=viewport( layout.pos.row=ii,
-                                                          layout.pos.col=seq(1+!show.rsep, ncol+1)))
-  }
-  if(show.vlines){
-    ## draw vertical lines, stopping or not before the names
-    for (jj in seq(2, ncol, 1)) 
+                                 layout.pos.col=seq(1+!show.rsep, ncol+1)))
+  ## draw vertical lines, stopping or not before the names
+  for (jj in seq(1, ncol, 1)) 
       grid.segments(1, 0, 1, 1, gp=gpar(col=separator), vp=viewport( layout.pos.col=jj,
-                                                          layout.pos.row=seq(1+!show.csep, nrow+1)))
-  }
-  ## draw box around the content (and the names if present)
+                                 layout.pos.row=seq(1+!show.csep, nrow+1)))
+  ## draw boxes around the content and the names
   if(show.box){
-    if(show.colnames & show.rownames){
-       grid.border(type=16, colour=separator,
+  grid.rect(gp=gpar(col=separator, fill=NA),
             vp=viewport( layout.pos.col=seq(2, ncol+1), layout.pos.row=seq(2, nrow+1)))
-     } else 
-    grid.rect(gp=gpar(col=separator), 
-                vp=viewport( layout.pos.col=seq(1, ncol+1), layout.pos.row=seq(1, nrow+1)))
+  grid.rect(gp=gpar(col=separator, fill=NA), 
+            vp=viewport( layout.pos.col=seq(2, ncol+1), layout.pos.row=1))
+  grid.rect(gp=gpar(col=separator, fill=NA),
+            vp=viewport( layout.pos.row=seq(2, nrow+1), layout.pos.col=1))
   }
-  
-  if(show.namesep){
-  ## draw corner, style depends on the presence of row/colnames
-    type <- if(show.colnames & show.rownames) 8 else
-    if(show.colnames & !show.rownames) 4 else
-    if(!show.colnames & show.rownames) 5 else 1
-    
-      grid.border(type=type, colour=separator,
-            vp=viewport( layout.pos.col=seq(2, ncol+1), layout.pos.row=seq(2, nrow+1)))
-  }
-  
-
 }
 
 
@@ -172,27 +160,25 @@ arrangeTableGrobs <- function(lgt, lgf, nrow, ncol, widths, heights,
 ##' @param d data.frame
 ##' @param rows vector of row names
 ##' @param cols vector of col names
+##' @param parse logical, parse labels as expressions
 ##' @param row.just justification of labels
 ##' @param col.just justification of labels
 ##' @param core.just justification of labels
 ##' @param separator colour of the border lines 
-##' @param show.box logical box surrounding the table
-##' @param show.vlines logical vertical lines
-##' @param show.hlines logical horizontal lines
-##' @param show.namesep logical draw lines to separate header(s)
-##' @param show.csep logical extend vert. separator to colnames 
-##' @param show.rsep logical extend vert. separator to rownames 
-##' @param equal.width logical 
+##' @param show.box logical
+##' @param show.csep logical separator for colnames 
+##' @param show.rsep logical separator for rownames 
+##' @param equal.width logical  
 ##' @param equal.height logical  
-##' @param padding.h unit of horizontal margin, per cell
-##' @param padding.v unit of vertical margin, per cell
+##' @param padding.h unit of horizontal margin,  per cell
+##' @param padding.v unit of vertical margin,  per cell
 ##' @param gpar.coretext gpar() for inner text
 ##' @param gpar.corefill gpar() for inner fill
 ##' @param gpar.coltext gpar() for colnames text
-##' @param h.odd.alpha numeric transparency factor for odd horizontal cells
-##' @param h.even.alpha numeric transparency factor for even horizontal cells
-##' @param v.odd.alpha numeric transparency factor for odd vertical cells
-##' @param v.even.alpha numeric transparency factor for even vertical cells
+##' @param h.odd.alpha h.odd.alpha 
+##' @param h.even.alpha h.even.alpha 
+##' @param v.odd.alpha v.odd.alpha 
+##' @param v.even.alpha v.even.alpha 
 ##' @param gpar.colfill gpar() for colnames fill
 ##' @param gpar.rowtext gpar() for rownames text
 ##' @param gpar.rowfill gpar() for rownames fill
@@ -228,12 +214,10 @@ arrangeTableGrobs <- function(lgt, lgf, nrow, ncol, widths, heights,
 ##' }
 
 
-tableGrob <- function(d, rows=rownames(d), cols=colnames(d),
+tableGrob <- function(d, rows=rownames(d), cols=colnames(d), parse=FALSE,
                       show.rownames=TRUE, show.colnames=TRUE,
                       row.just="center", col.just="center", core.just="center", 
-                      separator="white", show.box=FALSE,
-                      show.vlines=FALSE, show.hlines=FALSE, show.namesep=FALSE, 
-                      show.csep=FALSE, show.rsep=FALSE,
+                      separator="white", show.box=FALSE, show.csep=FALSE, show.rsep=FALSE,
                       equal.width = FALSE, equal.height=FALSE, 
                       padding.h = unit(4, "mm"), padding.v=unit(4, "mm"),
                       gp=NULL, 
@@ -253,8 +237,8 @@ tableGrob <- function(d, rows=rownames(d), cols=colnames(d),
 
   lg <- 
   with(theme, 
-       makeTableGrobs(as.character(as.matrix(d)), rows, cols,
-                        NROW(d), NCOL(d),
+       makeTableGrobs(as.character(as.matrix(d)), rows, cols, parse=parse,
+                        nrow(d), ncol(d), 
                         row.just = row.just, col.just = col.just, core.just = core.just, 
                         equal.width = equal.width, equal.height = equal.height, 
                         gpar.coretext = gpar.coretext,
@@ -267,11 +251,10 @@ tableGrob <- function(d, rows=rownames(d), cols=colnames(d),
                         gpar.colfill = gpar.colfill ))
   
   with(theme, # params provided as a list
-   gTree(lg=lg, d=d, rows= if(show.rownames) rows, cols=if(show.colnames) cols,
+   gTree(lg=lg, d=d, rows= if(show.rownames) rows, cols=if(show.colnames) cols, parse=parse,
          show.rownames=show.rownames, show.colnames=show.colnames,
          row.just = row.just, col.just = col.just, core.just = core.just, 
          separator=separator, show.box=show.box,
-         show.vlines=show.vlines, show.hlines=show.hlines, show.namesep=show.namesep, 
          show.csep=show.csep, show.rsep=show.rsep,
          equal.width = equal.width, equal.height = equal.height, 
          padding.h = padding.h, padding.v = padding.v, 
@@ -293,9 +276,9 @@ grid.table <- function(...)
 
 
 drawDetails.table <- function(x, recording=TRUE){
-
+  
   lg <- with(x, makeTableGrobs(as.character(as.matrix(d)), rows, cols,
-         NROW(d), NCOL(d),
+         nrow(d), ncol(d), parse,
          row.just = row.just, col.just = col.just, core.just = core.just, 
          equal.width = equal.width, equal.height = equal.height, 
          gpar.coretext = gpar.coretext,
@@ -321,12 +304,9 @@ drawDetails.table <- function(x, recording=TRUE){
   
   pushViewport(cells)
   tg <- arrangeTableGrobs(lg$lgt, lg$lgf, lg$nrow, lg$ncol, lg$widths, lg$heights,
-                          show.colnames=x$show.colnames, show.rownames=x$show.rownames, 
-                          padding.h = x$padding.h, padding.v = x$padding.v, 
-                          separator=x$separator, show.box=x$show.box,
-                          show.vlines=x$show.vlines, show.hlines=x$show.hlines,
-                          show.namesep=x$show.namesep, 
-                          show.csep=x$show.csep, show.rsep=x$show.rsep)
+          padding.h = x$padding.h, padding.v = x$padding.v, 
+          separator=x$separator, show.box=x$show.box,
+          show.csep=x$show.csep, show.rsep=x$show.rsep)
   upViewport()
 }
 widthDetails.table <- function(x){
@@ -364,20 +344,19 @@ updatelist <- function (x, y)
 ##' @return theme
 theme.default <- theme.grey <- function(...)
   updatelist(list(show.rownames=TRUE, show.colnames=TRUE,
-                  row.just="center", col.just="center", core.just="center", 
-                  separator="white", show.box=FALSE, show.vlines=FALSE, show.hlines=FALSE, show.namesep=FALSE,
-                  show.csep=FALSE, show.rsep=FALSE,
-                  equal.width = FALSE, equal.height=FALSE, 
-                  padding.h = unit(4, "mm"), padding.v=unit(4, "mm"),
-                  gp=NULL, 
-                  gpar.coretext = gpar(col="black", cex=1),
-                  gpar.coltext =  gpar(col="black", cex=1, fontface="bold"),
-                  gpar.rowtext =  gpar(col="black", cex=0.8, fontface="italic"),
-                  h.odd.alpha = 1, h.even.alpha = 1, 
-                  v.odd.alpha = 1, v.even.alpha = 1, 
-                  gpar.corefill = gpar(fill = "grey95", col="white"), 
-                  gpar.rowfill = gpar(fill = "grey90", col="white"), 
-                  gpar.colfill = gpar(fill = "grey90", col="white")), list(...))
+     row.just="center", col.just="center", core.just="center", 
+     separator="white", show.box=FALSE, show.csep=FALSE, show.rsep=FALSE,
+     equal.width = FALSE, equal.height=FALSE, 
+     padding.h = unit(4, "mm"), padding.v=unit(4, "mm"),
+     gp=NULL, 
+     gpar.coretext = gpar(col="black", cex=1),
+     gpar.coltext =  gpar(col="black", cex=1, fontface="bold"),
+     gpar.rowtext =  gpar(col="black", cex=0.8, fontface="italic"),
+     h.odd.alpha = 1, h.even.alpha = 1, 
+     v.odd.alpha = 1, v.even.alpha = 1, 
+     gpar.corefill = gpar(fill = "grey95", col="white"), 
+     gpar.rowfill = gpar(fill = "grey90", col="white"), 
+     gpar.colfill = gpar(fill = "grey90", col="white")), list(...))
 
 
 theme.list <- function(...)
